import os

import axipy
from PySide2.QtCore import QFile, QSize
from PySide2.QtUiTools import QUiLoader
from PySide2.QtWidgets import QWidget, QPushButton
from PySide2 import QtWidgets as qw
from axipy import action_manager

from FastPanael.ui.tool.helper_config import ConfigFiles


class WidgetConfig(QWidget):
    def __init__(self,path_folder,parent=None):
        super().__init__(parent)
        self.__path_folder=path_folder
        self.__ui=self.load_ui("widget_config_fast_panel_1.ui",parent)
        self.__curent_actions_name=[]
        self.__default_action=['OpenFile','SaveTables','Undo','Redo']
        #self.__initDefaultAction()
        self.__initFromFastPanel()
        self.__ui.pb_apply.clicked.connect(self.__update_fast_panel)
        self.__ui.pb_save.clicked.connect(self.__save_config)
        self.__ui.pb_set_default.clicked.connect(self.__set_default_config)
        self.__ui.pb_load.clicked.connect(self.__load_config)
        self.__name_config_current=None
    def load_ui(self,name_resource,parent):
        loader = QUiLoader()
        path = os.path.join(os.path.dirname(__file__),name_resource)
        ui_file = QFile(path)
        ui_file.open(QFile.ReadOnly)
        ui  = loader.load(ui_file,parent)
        ui_file.close()
        return ui
    def __initDefaultAction(self):
        for def_act in self.__default_action:
            self.__curent_actions_name.append(def_act)
            self.__addAction(def_act)
    def __initFromFastPanel(self):
        menu_bar = next(m for m in axipy.mainwindow.qt_object().findChildren(qw.QMenuBar) if
                        m.objectName() == "axiomaTitanRibbonBar")

        tool_bar = next(
            t for t in menu_bar.findChildren(qw.QToolBar) if t.objectName() == "qtn_Ribbon_Quick_Access_Bar")
        for action in tool_bar.actions():
            action_name=action.objectName()
            if action_name is None or action_name=='':
                continue
            self.__curent_actions_name.append(action_name)
            self.__addAction(action_name)
    def addAxiAction(self,axi_action_name):

        if axi_action_name in self.__default_action or axi_action_name in self.__curent_actions_name:
            return
        self.__curent_actions_name.append(axi_action_name)
        if len(self.__curent_actions_name)>len(self.__default_action):
            self.__ui.pb_apply.setEnabled(True)
            self.__ui.pb_save.setEnabled(True)
        else:
            self.__ui.pb_apply.setEnabled(False)
            self.__ui.pb_save.setEnabled(False)
        self.__addAction(axi_action_name)
    def __addAction(self,name_action):
        action=action_manager.get(name_action)
        if action is None:
            return
        icon = action.icon()
        if icon is None:
            bt_action = QPushButton(icon, action.text())
        else:
            bt_action = QPushButton(icon, '')
        bt_action.name_action = name_action
        bt_action.setFixedSize(QSize(40, 40))
        bt_action.setToolTip(action.text())
        bt_action.clicked.connect(self.__click_action)
        self.__ui.layout_actions.addWidget(bt_action)
    def __save_config(self):
        title = 'Сохранение файла конфигурации настроек'
        filter = 'Настройки панели быстрого доступа (*.fpconig)'
        folder = self.__path_folder
        #selected_filter = 'MapInfo Tab (*.tab)'
        dlg_save_res=axipy.save_file_dialog(filter,title,folder)
        if dlg_save_res is not None:
            isOk=ConfigFiles.save_config(dlg_save_res,self.__curent_actions_name)
            if isOk:
                self.__name_config_current=dlg_save_res.name
    def __set_default_config(self):
        self.__curent_actions_name.clear()
        self.__clearLayoutActions()
        self.__initDefaultAction()
        self.__ui.pb_apply.setEnabled(False)
        self.__ui.pb_save.setEnabled(False)
        ConfigFiles.setActiveConfig(self.__path_folder,None)
        self.__name_config_current=None
        self.__update_fast_panel()

    def __click_action(self):
        sender = self.sender()
        if isinstance(sender, QPushButton):
            name_action = sender.name_action
            if name_action in self.__default_action:
                return
            self.__clearUpdateLayoutActions(name_action)
    def __clearUpdateLayoutActions(self,name_action):
        self.__clearLayoutActions()
        self.__curent_actions_name.remove(name_action)
        if len(self.__curent_actions_name)<=len(self.__default_action):
            self.__ui.pb_apply.setEnabled(False)
            self.__ui.pb_save.setEnabled(False)
        for name in self.__curent_actions_name:
            self.__addAction(name)

    def __clearLayoutActions(self,name_action=None):
        while self.__ui.layout_actions.count():
            child = self.__ui.layout_actions.takeAt(0)
            if child.widget():
                if isinstance(child.widget(),QPushButton):
                    if name_action is not None:
                        name_=child.widget().name_action
                        if name_==name_action:
                            child.widget().clicked.disconnect()
                            child.widget().deleteLater()
                    else:
                        child.widget().clicked.disconnect()
                        child.widget().deleteLater()
    def __update_fast_panel(self):
        from PySide2 import QtWidgets as qw
        count_def = len(self.__default_action)
        if len(self.__curent_actions_name)<=count_def:
            self.__name_config_current=None
        ConfigFiles.setActiveConfig(self.__path_folder,self.__name_config_current)
        menu_bar = next(m for m in axipy.mainwindow.qt_object().findChildren(qw.QMenuBar) if
                        m.objectName() == "axiomaTitanRibbonBar")

        tool_bar = next(
            t for t in menu_bar.findChildren(qw.QToolBar) if t.objectName() == "qtn_Ribbon_Quick_Access_Bar")
        tool_bar.clear()

        for i,name_action in enumerate(self.__curent_actions_name):
            if i==count_def:
                tool_bar.addSeparator()
            tool_bar.addAction(axipy.action_manager[name_action])
    def __load_config(self):
        title = 'Открытие файла конфигурации настроек'
        filter = 'Настройки панели быстрого доступа (*.fpconig)'
        folder = self.__path_folder
        selected_filter = filter
        res = axipy.open_files_dialog(filter, title, folder, selected_filter)
        if res is None:
            return
        path_load_file=res[0]
        curent_actions_obj = ConfigFiles.readConfig(path_load_file.parent,path_load_file.name)
        if curent_actions_obj is None:
            return
        self.__name_config_current=path_load_file.name
        self.__curent_actions_name=[]
        self.__clearLayoutActions()
        for name_actions in curent_actions_obj['name_actions']:
            self.__curent_actions_name.append(name_actions)
            self.__addAction(name_actions)
        self.__ui.pb_apply.setEnabled(True)


    def show(self):
        self.__ui.show()